﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Threading;
using Produmex.Foundation;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.Reflection;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation.Utilities;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    public class WorkflowScript_MainReceptionWSBScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Private fields
        private CultureInfo m_defaultCultureInfo;
        private PmxOseCompany m_pmxOseCompany;

        // Input parameters
        // <none>

        // Output parameters
        // <none>

        // Sub-flows
        private WorkflowInstanceBase m_logOnFlow;
        private bool m_logOnFlow_LogOnCanceled;

        // Menu options
        private class MenuOption
        {
            public string ButtonKey;
            public WorkflowInstanceBase Flow;

            public MenuOption(string buttonKey, WorkflowInstanceBase flow)
            {
                ButtonKey = buttonKey;
                Flow = flow;
            }
        };
        private List<MenuOption> m_MenuOptions = new List<MenuOption>();

        public WorkflowScript_MainReceptionWSBScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
            // Initialize required sub-flows
            m_logOnFlow = Create("LogOnScript");
            m_logOnFlow.BindParameters(
                new ReadWriteBinder<CultureInfo>("DefaultCultureInfo", this, "m_defaultCultureInfo"),
                new ReadWriteBinder<PmxOseCompany>("PmxOseCompany", this, "m_pmxOseCompany"),
                new ReadWriteBinder<bool>("LogOnCanceled", this, "m_logOnFlow_LogOnCanceled")
            );

            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_RECEPTION", Create("ReceptionWSBScript")));


            foreach (MenuOption menuOption in m_MenuOptions)
            {
                List<string> list = new List<string>();
                list.Add(menuOption.ButtonKey);
                menuOption.Flow.BindParameters(
                    new ReadOnlyBinder<CultureInfo>("DefaultCultureInfo", this, "m_defaultCultureInfo"),
                    new ReadOnlyBinder<PmxOseCompany>("PmxOseCompany", this, "m_pmxOseCompany")
                );
            }

        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Store menu options indexed by button key
            Dictionary<string, MenuOption> menuOptionsByButtonKey = new Dictionary<string, MenuOption>();
            foreach (MenuOption menuOption in m_MenuOptions)
            {
                menuOptionsByButtonKey[menuOption.ButtonKey] = menuOption;
            }

            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;

            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            while (true)
            {
            Step_Logon:

                // Run sub-flow LogOn
                base.StartSubFlowAndWait(m_logOnFlow, true);


                if (m_logOnFlow_LogOnCanceled) break;

                string initialErrorKey = null;

                int topMenuOptionIndex = 0;

            Step_SelectFunction:

                string[] buttonKeys = new string[6];
                int btnIdx = 0;
                for (int i = topMenuOptionIndex; i < m_MenuOptions.Count; i++)
                {
                    buttonKeys[btnIdx] = m_MenuOptions[i].ButtonKey;

                    if (++btnIdx >= 4) break;
                }

                ToolBarButtons toolBarButtons = ToolBarButtons.Stop;
                if (topMenuOptionIndex > 0) toolBarButtons |= ToolBarButtons.Back;
                if (topMenuOptionIndex + 4 < m_MenuOptions.Count) toolBarButtons |= ToolBarButtons.Forward;

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                    m_defaultCultureInfo, BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_FUNCTION" + (topMenuOptionIndex / 4 + 1).ToString(),
                    "AllowedButtons", buttonKeys,
                    "EnabledToolBarButtons", toolBarButtons,
                    "ScreenType", ScreenType.Menu
                    ));
                Message msg = WaitForMessage();
                initialErrorKey = null;
                if (msg.Name.EndsWith(".ButtonClicked"))
                {
                    string buttonKey = ExtractParameter<string>(msg.Parameters, "buttonKey");
                    MenuOption menuOption;
                    if (menuOptionsByButtonKey.TryGetValue(buttonKey, out menuOption))
                    {
                        try
                        {
                            base.StartSubFlowAndWait(menuOption.Flow, true);

                        }
                        catch (Exception ex)
                        {
                            s_log.Error(String.Format("Exception in workflow behind button {0}", menuOption.ButtonKey), ex);

                            ProdumexError pmxError;
                            ex = ExtractExceptionAndPmxErrorForDisplay(ex, out pmxError);

                            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                m_defaultCultureInfo, BuildParamCollection(
                                "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, m_defaultCultureInfo),
                                "NoTranslationOfMessageKey", true,
                                "PmxError", pmxError,
                                "MessageAddendum", ReflectionHelper.CompactStackTrace(ex.StackTrace),
                                "ShowButton", true,
                                "IsError", true
                                ));
                            msg = WaitForMessage();
                        }

                        goto Step_SelectFunction;
                    }
                }
                if (msg.Name.EndsWith(".BackRequested"))
                {
                    topMenuOptionIndex = Math.Max(0, topMenuOptionIndex - 4);
                    goto Step_SelectFunction;
                }
                if (msg.Name.EndsWith(".ForwardRequested"))
                {
                    topMenuOptionIndex += 4;
                    goto Step_SelectFunction;
                }
                if (msg.Name.EndsWith(".StopRequested"))
                {
                    sboProviderService.LogOffWithSessionId(session.Id);
                    goto Step_Logon;
                }

                initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
                goto Step_SelectFunction;
            }
        }

        #endregion
    }
}
